@@ -1,7 +1,7 @@
 """A type-hinted Entity Component System based on Python dictionaries and sets."""
 from __future__ import annotations
 
-__version__ = "0.1.0"
+__version__ = "0.2.0"
 from functools import partial
 from typing import (
     AbstractSet,
@@ -10,6 +10,7 @@ from typing import (
     Final,
     Iterable,
     Iterator,
+    Mapping,
     MutableMapping,
     MutableSet,
     Tuple,
@@ -39,7 +40,7 @@ def abstract_component(cls: type[T]) -> type[T]:
 class Entity:
     """A unique entity in a world."""
 
-    __slots__ = ("world", "__weakref__")
+    __slots__ = ("world",)
 
     def __init__(self, world: World) -> None:
         """Initialize a new unique entity."""
@@ -66,6 +67,35 @@ class Entity:
         """Access an entities many-to-many relations."""
         return EntityRelations(self)
 
+    @property
+    def name(self) -> object:
+        """The unique name of this entity or None.
+
+        You may assign a new name, but if an entity of the world already has that name then it will lose it.
+        """
+        return self.world._names_by_entity.get(self)
+
+    @name.setter
+    def name(self, value: object) -> None:
+        old_name = self.name
+        if old_name is not None:  # Remove self from names.
+            del self.world._names_by_name[old_name]
+            del self.world._names_by_entity[self]
+
+        if value is not None:  # Add self to names.
+            old_entity = self.world._names_by_name.get(value)
+            if old_entity is not None:  # Remove entity with old name, name will be overwritten.
+                del self.world._names_by_entity[old_entity]
+            self.world._names_by_name[value] = self
+            self.world._names_by_entity[self] = value
+
+    def __repr__(self) -> str:
+        """Return a representation of this entity."""
+        items = [self.__class__.__name__]
+        name = self.name
+        items.append(f"0x{id(self):X}" if name is None else f"name={name!r}")
+        return f"<{' '.join(items)}>"
+
 
 class EntityComponents(MutableMapping[Union[Type[Any], Tuple[object, Type[Any]]], Any]):
     """A proxy attribute to access an entities components like a dictionary."""
@@ -105,10 +135,12 @@ class EntityComponents(MutableMapping[Union[Type[Any], Tuple[object, Type[Any]]]
     def __delitem__(self, key: type[object] | tuple[object, type[object]]) -> None:
         """Delete a component from an entity."""
         assert self.__assert_key(key)
+
         del self.entity.world._components_by_type[key][self.entity]
-        self.entity.world._components_by_entity[self.entity].remove(key)
         if not self.entity.world._components_by_type[key]:
             del self.entity.world._components_by_type[key]
+
+        self.entity.world._components_by_entity[self.entity].remove(key)
         if not self.entity.world._components_by_entity[self.entity]:
             del self.entity.world._components_by_entity[self.entity]
 
@@ -147,9 +179,10 @@ class EntityTags(MutableSet[Any]):
     def discard(self, tag: object) -> None:
         """Discard a tag from an entity."""
         self.entity.world._tags_by_entity[self.entity].discard(tag)
-        self.entity.world._tags_by_key[tag].discard(self.entity)
         if not self.entity.world._tags_by_entity[self.entity]:
             del self.entity.world._tags_by_entity[self.entity]
+
+        self.entity.world._tags_by_key[tag].discard(self.entity)
         if not self.entity.world._tags_by_key[tag]:
             del self.entity.world._tags_by_key[tag]
 
@@ -180,28 +213,32 @@ class EntityRelationsMapping(MutableSet[Entity]):
     def add(self, target: Entity) -> None:
         """Add a relation target to this key."""
         world = self.entity.world
-        world._relations_by_key[self.key][None].add(target)
         world._relations_by_key[self.key][self.entity].add(target)
-        world._relations_by_target[(self.key, None)].add(self.entity)
+        world._relations_by_key[self.key][None].add(target)
         world._relations_by_target[(self.key, target)].add(self.entity)
+        world._relations_by_target[(self.key, None)].add(self.entity)
 
     def discard(self, target: Entity) -> None:
         """Discard a relation target from this key."""
         world = self.entity.world
+
         world._relations_by_key[self.key][self.entity].discard(target)
-        world._relations_by_target[(self.key, target)].discard(self.entity)
         if not world._relations_by_key[self.key][self.entity]:
             del world._relations_by_key[self.key][self.entity]
-            world._relations_by_key[self.key][None].discard(target)
-            if not world._relations_by_key[self.key][None]:
-                del world._relations_by_key[self.key][None]
+
+        world._relations_by_key[self.key][None].discard(target)
+        if not world._relations_by_key[self.key][None]:
+            del world._relations_by_key[self.key][None]
             if not world._relations_by_key[self.key]:
                 del world._relations_by_key[self.key]
+
+        world._relations_by_target[(self.key, target)].discard(self.entity)
         if not world._relations_by_target[(self.key, target)]:
             del world._relations_by_target[(self.key, target)]
-            world._relations_by_target[(self.key, None)].discard(self.entity)
-            if not world._relations_by_target[(self.key, None)]:
-                del world._relations_by_target[(self.key, None)]
+
+        world._relations_by_target[(self.key, None)].discard(self.entity)
+        if not world._relations_by_target[(self.key, None)]:
+            del world._relations_by_target[(self.key, None)]
 
     def __contains__(self, target: Entity) -> bool:  # type: ignore[override]
         """Return True if this relation contains the given value."""
@@ -252,7 +289,7 @@ class EntityRelations(MutableMapping[object, EntityRelationsMapping]):
 
     def __len__(self) -> int:
         """Return the number of relations this entity has."""
-        return len(list(self))  # Slow!
+        return len(list(self.__iter__()))  # Slow!
 
 
 class EntityRelationsExclusive(MutableMapping[object, Entity]):
@@ -304,23 +341,45 @@ class World:
 
     def __init__(self) -> None:
         """Initialize a new world."""
-        # Spare components as `v[ComponentType][Entity] = component`
+        # Spare-set components.
+        # ComponentKey is `type` or `(tag, type)`, see annotation.
+        # dict[ComponentKey][Entity] = component_instance
         self._components_by_type: DefaultDict[_ComponentKey[object], dict[Entity, Any]] = DefaultDict(dict)
-        # Components types belonging to entities.
+        # dict[Entity] = {component_keys_owned_by_entity}
         self._components_by_entity: DefaultDict[Entity, set[_ComponentKey[object]]] = DefaultDict(set)
 
+        # Sparse-set tags.
+        # dict[tag] = {all_entities_with_tag}
         self._tags_by_key: DefaultDict[object, set[Entity]] = DefaultDict(set)
+        # dict[Entity] = {all_tags_for_entity}
         self._tags_by_entity: DefaultDict[Entity, set[Any]] = DefaultDict(set)
 
+        # Sparse-set relations.
+        # dict[tag][this_entity] = {target_entities_for_entity}
+        # dict[tag][None] = {target_entities_for_tag}
         self._relations_by_key: DefaultDict[object, DefaultDict[Entity | None, set[Entity]]] = DefaultDict(
             partial(DefaultDict, set)  # type: ignore[arg-type]
         )
+        # dict[(tag, this_entity)] = {target_entities_for_entity}
+        # dict[(tag, None)] = {target_entities_for_tag}
         self._relations_by_target: DefaultDict[tuple[object, Entity | None], set[Entity]] = DefaultDict(set)
 
+        # Named objects dictionary.
+        # dict[name] = named_entity
+        self._names_by_name: dict[object, Entity] = {}
+        # dict[Entity] = entities_name
+        self._names_by_entity: dict[Entity, object] = {}
+
+    @property
+    def named(self) -> Mapping[object, Entity]:
+        """A view into this worlds named entities."""
+        return self._names_by_name
+
     def new_entity(
         self,
         components: Iterable[object] = (),
         *,
+        name: object = None,
         tags: Iterable[Any] = (),
     ) -> Entity:
         """Create and return a new entity."""
@@ -329,6 +388,7 @@ class World:
         entity_tags = entity.tags
         for tag in tags:
             entity_tags.add(tag)
+        entity.name = name
         return entity
 
     @property
@@ -458,10 +518,10 @@ class Query:
 
         entities = list(self._get_entities(set(key) - {Entity}))
         entity_components = []
-        for component_type in key:
-            if component_type is Entity:
+        for component_key in key:
+            if component_key is Entity:
                 entity_components.append(entities)
                 continue
-            world_components = self.world._components_by_type[component_type]
+            world_components = self.world._components_by_type[component_key]
             entity_components.append([world_components[entity] for entity in entities])
         return zip(*entity_components)
